package com.diagbot.facade;

//import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.api.ImportExcel;
import com.diagbot.dto.ConceptDTO;
import com.diagbot.entity.*;
//import com.diagbot.dto.RelationDTO;
//import com.diagbot.exception.CommonErrorCode;
//import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.ConceptServiceImpl;
import com.diagbot.vo.AmendTermVo;
//import com.diagbot.vo.MedicalVo;
import com.diagbot.vo.TermVo;
//import org.apache.poi.ss.usermodel.Row;
//import org.apache.poi.ss.usermodel.Cell;
//import org.apache.poi.hssf.usermodel.HSSFWorkbook;
//import org.apache.poi.ss.usermodel.Sheet;
//import org.apache.poi.ss.usermodel.Workbook;
//import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.multipart.MultipartFile;

//import java.io.IOException;
//import java.io.InputStream;
import java.time.LocalDateTime;
//import java.util.ArrayList;
import java.util.ArrayList;
import java.util.HashMap;
//import java.util.List;
import java.util.List;
import java.util.Map;


/**
 * @Description: 术语查询业务层
 * @author: Weixuan Huang
 * @time: 2019/1/14 16:17
 */
@Component
public class ConceptFacade extends ConceptServiceImpl {

    @Autowired
    private LibraryInfoFacade libraryinfoFacade;
    @Autowired
    private LibraryDetailFacade libraryDetailFacade;
    @Autowired
    private MedicalFacade medicalFacade;
    @Autowired
    private RelationFacade relationFacade;


    /**
     * 获取医学术语信息
     *
     * @param termvo
     * @return ConceptDTO
     */
    public ConceptDTO getConceptInfo(@RequestBody TermVo termvo) {

        LibraryInfo libraryInfo;
        ConceptDTO conceptDTO = getConcept(termvo);
        // 获取与医学术语的基本信息
        if (conceptDTO != null) {

            Map<String, Object> params = new HashMap<>();
            params.put("id", conceptDTO.getLibId());
            params.put("concept_id", conceptDTO.getId());
            libraryInfo = libraryinfoFacade.getLibraryInfor(params);

            if (libraryInfo.getIsConcept() != null && libraryInfo.getIsConcept() == 1) {
                conceptDTO.setName(libraryInfo.getName());
            }
            conceptDTO.setLibraryInfo(libraryInfo);

            // 获取与术语相关的医学知识
            params = new HashMap<>();
            params.put("concept_id", conceptDTO.getId());
            conceptDTO.setInformation(libraryDetailFacade.getLibraryDetails(params));
            // 获取与术语相关的临床医学信息
            params = new HashMap<>();
            params.put("name", termvo.getTerm());
            params.put("concept_id", conceptDTO.getId());
            conceptDTO.setMedicalInfo(medicalFacade.getMedicalInfo(params));
            // 获取术语的所有上级信息
            conceptDTO.setRelations(relationFacade.getAllRelation(termvo));
        }

        return conceptDTO;
    }

    /**
     * 获取医学术语列表
     *
     * @param
     * @return List<ConceptDTO>
     */
    public List<ConceptDTO> getAllConcept() {
        List<ConceptDTO> conceptDTOS = new ArrayList<>();
        ConceptDTO conceptDTO;
        LibraryInfo libraryInfo;

        List<Concept> concepts = this.list();

        for (Concept concept : concepts) {
            conceptDTO = new ConceptDTO();

            conceptDTO.setId(concept.getId());
            conceptDTO.setLibId(concept.getLibId());

            Map<String, Object> params = new HashMap<>();
            params.put("id", concept.getLibId());
            params.put("concept_id", concept.getId());
            params.put("is_concept", 1);
            libraryInfo = libraryinfoFacade.getLibraryInfor(params);

            if (libraryInfo != null) {
                conceptDTO.setName(libraryInfo.getName());
                conceptDTO.setLibraryInfo(libraryInfo);

                conceptDTOS.add(conceptDTO);
            }
        }

        return conceptDTOS;
    }


        /**
         * 更新医学术语信息
         *
         * @param amendTermVo
         * @return ConceptDTO
         */
    public ConceptDTO upsertConceptInfo(@RequestBody AmendTermVo amendTermVo) {
        ConceptDTO conceptDTO = new ConceptDTO();

        TermVo termVo = new TermVo();
        termVo.setTerm(amendTermVo.getConcept());
        termVo.setType(amendTermVo.getType());

        // 如果概念术语不存在，添加新术语，然后添加新的概念
        conceptDTO = getConcept(termVo);
        if (conceptDTO == null) {
            AmendTermVo amendconcept = new AmendTermVo();
            amendconcept.setOldterm(amendTermVo.getOldterm());
            amendconcept.setNewterm(amendTermVo.getConcept());
            amendconcept.setConcept("");
            amendconcept.setType(amendTermVo.getType());
            libraryinfoFacade.upsertLibraryInfo(amendconcept);

            Map<String, Object> params = new HashMap<>();
            params.put("name", amendTermVo.getConcept());
            params.put("type", amendTermVo.getType());
            int libid = libraryinfoFacade.getLibraryInfor(params).getId().intValue();


            Concept concept = new Concept();
            concept.setLibId(libid);
            concept.setGmtCreated(LocalDateTime.now());
            this.saveOrUpdate(concept);

            termVo.setType(amendTermVo.getType());
            conceptDTO = getConcept(termVo);
        }

        amendTermVo.setConcept_id(conceptDTO.getId());
        LibraryInfo libraryInfo = libraryinfoFacade.upsertLibraryInfo(amendTermVo);
        conceptDTO.setLibraryInfo(libraryInfo);

        conceptDTO = updateConceptInfo(amendTermVo, conceptDTO);

        return conceptDTO;
    }

    /**
     * 更新或添加医学术语信息
     *
     * @param amendTermVo
     * @param conceptDTO
     * @return ConceptDTO
     */
    public ConceptDTO updateConceptInfo(AmendTermVo amendTermVo, ConceptDTO conceptDTO) {

        libraryDetailFacade.upsertLibraryInfo(amendTermVo);
        conceptDTO.setInformation(amendTermVo.getInformation());

        medicalFacade.upsertMedicalInfo(amendTermVo);
        conceptDTO.setMedicalInfo(amendTermVo.getMedicalInfo());

        relationFacade.upsertRelationInfo(amendTermVo);
        conceptDTO.setRelations(amendTermVo.getRelations());

        return conceptDTO;
    }



    /**
     * 获取概念Id
     *
     * @param term
     * @return Integer
     */
    public int getConceptId(String term, String type) {
        int id = 0;
        TermVo termVo = new TermVo();

        try {
            termVo.setTerm(term);
            termVo.setType(type);
            ConceptDTO conceptDTO = getConcept(termVo);
            if (conceptDTO != null) {
                id = conceptDTO.getId();
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        finally {
            return id;
        }
    }

    /**
     * 从文件批量导入术语信息
     *
     * @param file
     * @return ConceptDTO
     */
    public ConceptDTO importBatch(MultipartFile file) {
        ImportExcel.setConceptFacade(this);
        ImportExcel.setLibraryinfoFacade(libraryinfoFacade);
        ImportExcel.setLibraryDetailFacade(libraryDetailFacade);
        ImportExcel.setMedicalFacade(medicalFacade);
        ImportExcel.setRelationFacade(relationFacade);

        ConceptDTO conceptDTO = ImportExcel.importBatch(file);

        return conceptDTO;
    }


}
