package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.ConceptDTO;
import com.diagbot.entity.LibraryInfo;
import com.diagbot.entity.Medical;
import com.diagbot.service.impl.LibraryInfoServiceImpl;
import com.diagbot.vo.AmendTermVo;
import com.diagbot.vo.TermVo;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 术语基本信息查询业务层
 * @author: Weixuan Huang
 * @time: 2019/1/16 14:17
 */
@Component
public class LibraryInfoFacade extends LibraryInfoServiceImpl {

    private DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd hh:mm:ss");


    /**
     * 获取医学术语基本信息
     *
     * @param params
     * @result LibraryInfo
     */
    public LibraryInfo getLibraryInfor(Map<String, Object> params) {
        QueryWrapper<LibraryInfo> wrapper = new QueryWrapper();
        LibraryInfo libraryinfor = null;

        for (String key:params.keySet()) {
            wrapper.eq(key, params.get(key));
        }

        libraryinfor = this.getOne(wrapper);

        return libraryinfor;
    }


    /**
     * 更新或添加医学术语基本信息
     *
     * @param amendTermVo
     * @return LibraryInfo
     */
    public LibraryInfo upsertLibraryInfo(AmendTermVo amendTermVo) {

        String oldterm = amendTermVo.getOldterm();
        String newterm = amendTermVo.getNewterm();
        String type = amendTermVo.getType();
        String concept = amendTermVo.getConcept();
        int concept_id = amendTermVo.getConcept_id();


        Map<String, Object> map = new HashMap<>();
        map.put("name", concept);
        map.put("type", type);
        LibraryInfo conceptInfo = this.getLibraryInfor(map);

        map.put("name", newterm);
        LibraryInfo libraryInfo = getLibraryInfor(map);

        // 如果术语概念存在，而且新术语不存在
        if (libraryInfo == null) {

            // 如果原先术语名称为空，则添加新术语
            if (oldterm.trim().length() == 0 && newterm.trim().length() > 0) {
                libraryInfo = new LibraryInfo();
                libraryInfo.setGmtCreated(LocalDateTime.now());
            }
            // 原先的术语名称不为空，则修改原先的术语
            else if (oldterm.trim().length() > 0 && !oldterm.trim().equals(newterm.trim())) {
                map.put("name", oldterm);
                libraryInfo = getLibraryInfor(map);

                libraryInfo.setGmtModified(LocalDateTime.now());
            }

            libraryInfo.setName(newterm);
            libraryInfo.setType(type);
        }

        if (conceptInfo != null) {
            libraryInfo.setConceptId(concept_id);

            if (newterm.trim().equals(concept.trim())) {
                libraryInfo.setIsConcept(1);
            }
        }

        this.saveOrUpdate(libraryInfo);

        return libraryInfo;
    }


}
