package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.TypeDTO;
import com.diagbot.entity.Medical;
import com.diagbot.service.impl.MedicalServiceImpl;
import com.diagbot.vo.AmendTermVo;
import com.diagbot.vo.MedicalVo;
import com.diagbot.vo.TypeVo;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 术语临床信息查询业务层
 * @author: Weixuan Huang
 * @time: 2019/1/16 16:07
 */
@Component
public class MedicalFacade extends MedicalServiceImpl {

    /**
     * 获取医学术语知识
     *
     * @param params
     * @result List<Medical>
     */
    public List<Medical> getMedicalInfo(Map<String, Object> params) {
        QueryWrapper<Medical> wrapper = new QueryWrapper<>();
        List<Medical> medicals;

        for (String key:params.keySet()) {
            wrapper.eq(key, params.get(key));
        }

        medicals = this.list(wrapper);

        return medicals;
    }

    /**
     * 更新或添加医学术语信息
     *
     * @param amendTermVo
     * @return
     */
    public void upsertMedicalInfo(AmendTermVo amendTermVo) {
        Map<String, Object> map;
        Medical medical;
        List<Medical> medicals;

        if (amendTermVo.getMedicalInfo() != null && amendTermVo.getMedicalInfo().size() > 0) {
            for (int i=0; i<amendTermVo.getMedicalInfo().size(); i++) {
                medical = amendTermVo.getMedicalInfo().get(i);

                map = new HashMap<>();
                map.put("name", amendTermVo.getNewterm());
                medicals = getMedicalInfo(map);

                if (medicals.size() > 0) {
                    Medical exist_medical = getMedicalInfo(map).get(0);
                    /*
                    // 如果更新数据有效
                    if (medical.getConceptId() != null) {
                        medical.setId(exist_medical.getId());
                    }
                    else {
                        medical = exist_medical;
                        medical.setName(amendTermVo.getNewterm());
                    }
                    */
                    medical.setId(exist_medical.getId());
                }
                else {
                    medical.setConceptId(amendTermVo.getConcept_id());
                }


                this.saveOrUpdate(medical);
                amendTermVo.getMedicalInfo().set(i, medical);
            }
        }
    }


    /**
     * 获取术语类型信息
     *
     * @param type
     * @return TypeDTO
     */
    public TypeDTO getTypeInfo(String type) {
        TypeDTO typeDTO = null;
        TypeVo typeVo = new TypeVo();

        try {
            typeVo.setType(type);
            List<TypeDTO> typelist = getTypeId(typeVo);

            if (typelist.size() > 0) {
                typeDTO = getTypeId(typeVo).get(0);
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        finally {
            return typeDTO;
        }
    }

    /**
     * 清空医学术语信息表
     *
     */
    public void deleteMedical() {
        truncateMedical();
    }
}
