package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ConceptDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.TermDTO;
import com.diagbot.facade.ConceptFacade;
import com.diagbot.vo.AmendTermVo;
import com.diagbot.vo.TermVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-30
 */
@RestController
@RequestMapping("/concept")
@Api(value = "医学标准术语API", tags = { "医学标准术语API" })

public class ConceptController {
    @Autowired
    private ConceptFacade conceptFacade;


    @ApiOperation(value = "查询所有术语")
    @PostMapping("getAllConcept")
    @SysLogger("getAllConcept")
    public RespDTO<TermDTO> getAllConcept() {
        List<ConceptDTO> data = conceptFacade.getAllConcept();

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "术语信息查询",
            notes = "Term：术语名称, 必填<br>Type：术语类型, 必填<br>")
    @PostMapping("/getConceptInfo")
    @SysLogger("getConceptInfo")
    public RespDTO<TermDTO> getConceptInfo(@Valid @RequestBody TermVo termvo) {
        ConceptDTO data = conceptFacade.getConceptInfo(termvo);

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "术语修改",
            notes = "OldTerm: 原来的术语名称, 必填<br/>NewTerm: 新的术语名称, 必填<br/>" +
                    "Concept: 概念名称，必填<br/>Type: 新的术语类型, 必填<br/>")
    @PostMapping("/updateConceptInfo")
    @SysLogger("updateConceptInfo")
    public RespDTO<ConceptDTO> updateConceptInfo(@Valid @RequestBody AmendTermVo amendTermVo) {
        ConceptDTO data = conceptFacade.upsertConceptInfo(amendTermVo);

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "术语导入", notes = "导入术语信息")
    @PostMapping("/importTermExcel")
    public RespDTO<ConceptDTO> lisExcelAnalysis(@RequestParam("uploadfile") MultipartFile file) {
        ConceptDTO data = conceptFacade.importBatch(file);

        return RespDTO.onSuc(data);
    }

}
